"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Web3 = require("web3");
const data_packet_service_1 = require("../service/data-packet-service");
const MinespiderPacketStorage_1 = require("../contracts/MinespiderPacketStorage");
const Minespider_1 = require("../contracts/Minespider");
const SILQToken_1 = require("../contracts/SILQToken");
class Web3Adapter {
    constructor(provider) {
        this.provider = provider;
        this.initiated = false;
        this.web3 = new Web3(provider);
    }
    wasInitiated() {
        return this.initiated;
    }
    init(minespiderContractAddress, dataPacketStorageAddress, silqTokenAddress) {
        this.minespiderContractAddress = minespiderContractAddress;
        this.minespider = (new this.web3.eth.Contract(Minespider_1.default, minespiderContractAddress));
        this.minespiderPacketManager = (new this.web3.eth.Contract(MinespiderPacketStorage_1.default, dataPacketStorageAddress));
        this.silqToken = (new this.web3.eth.Contract(SILQToken_1.default, silqTokenAddress));
        this.initiated = true;
    }
    async registerEntity(entity) {
        let transactionPromise = this.minespider.methods.createEntity(entity.id, entity.entityType, entity.name, entity.latitude, entity.longitude, entity.location)
            .send({
            from: (await this.web3.eth.getAccounts())[0]
        });
        let receipt = await this.awaitTransactionConfirmation(transactionPromise);
        return receipt.transactionHash;
    }
    async modifyMassBalanceCertification(producerId, materialId, cmw) {
        let transactionPromise = this.minespider
            .methods
            .modifyMassBalanceCertification(producerId, materialId, cmw)
            .send({
            from: (await this.web3.eth.getAccounts())[0]
        });
        let receipt = await this.awaitTransactionConfirmation(transactionPromise);
        return receipt.transactionHash;
    }
    async certifyDataPacket(packetId, cmw, materialId, publicFilesAddresses, privateFilesAddresses) {
        let pricePerCmw = await this.minespiderPacketManager.methods.getSilqPerCmwPrice().call();
        let silqToTransfer = pricePerCmw * cmw;
        await this.silqToken.methods.approve(this.minespiderContractAddress, Web3.utils.toWei(String(silqToTransfer))).send({
            from: (await this.web3.eth.getAccounts())[0]
        });
        let transactionPromise = this.minespiderPacketManager
            .methods
            .certifyDataPacket(packetId, cmw, materialId, JSON.stringify(publicFilesAddresses), JSON.stringify(privateFilesAddresses))
            .send({
            from: (await this.web3.eth.getAccounts())[0]
        });
        await this.awaitTransactionConfirmation(transactionPromise);
        return packetId;
    }
    async getPacket(packetId) {
        let packet = await this.minespiderPacketManager.methods.getPacket(packetId).call();
        if (packet._id == '0x0000000000000000000000000000000000000000') {
            throw new Error('Packet not found');
        }
        var status = "";
        switch (parseInt(packet.status)) {
            case 0: {
                status = "Created";
                break;
            }
            case 1: {
                status = "Sold";
                break;
            }
            case 2: {
                status = "Purchased";
                break;
            }
        }
        return new data_packet_service_1.Packet(packet._id, packet.cmw, packet.materialId, JSON.parse(packet.publicFiles), JSON.parse(packet.privateFiles), status);
    }
    async sellPacket(packetId, newPacketId, newOwnerAddress, cmw, publicFilesAddresses, privateFilesAddresses) {
        let pricePerCmw = await this.minespiderPacketManager
            .methods
            .getSilqPerCmwPrice()
            .call();
        let silqToTransfer = pricePerCmw * cmw;
        let approveTransactionPromise = this.silqToken
            .methods
            .approve(this.minespiderContractAddress, Web3.utils.toWei(String(silqToTransfer)))
            .send({
            from: (await this.web3.eth.getAccounts())[0]
        });
        await this.awaitTransactionConfirmation(approveTransactionPromise);
        let sellPacketTransactionPromise = this.minespiderPacketManager
            .methods
            .sellPacket(packetId, newPacketId, newOwnerAddress, JSON.stringify(publicFilesAddresses), JSON.stringify(privateFilesAddresses))
            .send({
            from: (await this.web3.eth.getAccounts())[0]
        });
        await this.awaitTransactionConfirmation(sellPacketTransactionPromise);
        return newPacketId;
    }
    async splitAndSellPacket(packetId, newOwnerAddress, part1Id, part2Id, amountToSell, publicFilesAddresses, privateFilesAddresses) {
        return new Promise(async (resolve, reject) => {
            try {
                let pricePerCmw = await this.minespiderPacketManager
                    .methods
                    .getSilqPerCmwPrice()
                    .call();
                let silqToTransfer = pricePerCmw * amountToSell;
                let approveTransactionPromise = this.silqToken
                    .methods
                    .approve(this.minespiderContractAddress, Web3.utils.toWei(String(silqToTransfer)))
                    .send({
                    from: (await this.web3.eth.getAccounts())[0]
                });
                await this.awaitTransactionConfirmation(approveTransactionPromise);
                let sellPacketTransactionPromise = this.minespiderPacketManager
                    .methods
                    .splitAndSellPacket(packetId, newOwnerAddress, part1Id, part2Id, amountToSell, JSON.stringify(publicFilesAddresses), JSON.stringify(privateFilesAddresses))
                    .send({
                    from: (await this.web3.eth.getAccounts())[0]
                });
                await this.awaitTransactionConfirmation(sellPacketTransactionPromise);
                resolve(part1Id);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async updateSilqTokenAddress(newAddress) {
        return new Promise(async (resolve, reject) => {
            try {
                let transactionPromise = this.minespider
                    .methods
                    .updateSilqTokenAddress(newAddress)
                    .send({
                    from: (await this.web3.eth.getAccounts())[0]
                });
                await this.awaitTransactionConfirmation(transactionPromise);
                resolve(newAddress);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async updateMinespiderAddress(newAddress) {
        return new Promise(async (resolve, reject) => {
            try {
                let transactionPromise = this.minespiderPacketManager
                    .methods
                    .updateMinespiderAddress(newAddress)
                    .send({
                    from: (await this.web3.eth.getAccounts())[0]
                });
                await this.awaitTransactionConfirmation(transactionPromise);
                resolve(newAddress);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async updateDataPacketStorageAddress(newAddress) {
        return new Promise(async (resolve, reject) => {
            try {
                await this.minespider
                    .methods
                    .updateDataPacketStorageAddress(newAddress)
                    .send({
                    from: (await this.web3.eth.getAccounts())[0]
                });
                resolve(newAddress);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async getCurrentAccountAddress() {
        let accounts = await this.web3.eth.getAccounts();
        return accounts[0];
    }
    async getSILQBalance(address) {
        let silqBalance = await this.silqToken.methods.balanceOf(address).call();
        return Number(silqBalance);
    }
    async getProducedMassBalance(producerId, materialId) {
        let producedMassBalance = await this.minespiderPacketManager.methods.getProducedMassBalance(producerId, materialId).call();
        return Number(producedMassBalance);
    }
    async getOwnedMassBalance(producerId, materialId) {
        let ownedMassBalance = await this.minespiderPacketManager.methods.getMassBalance(producerId, materialId).call();
        return Number(ownedMassBalance);
    }
    async getCertifiedMassBalance(producerId, materialId) {
        let certifiedMassBalance = await this.minespider.methods.getCertifiedMassBalance(producerId, materialId).call();
        return Number(certifiedMassBalance);
    }
    async getTransactionHash(web3Promise) {
        return new Promise((resolve, reject) => {
            try {
                web3Promise.on('transactionHash', function (hash) {
                    resolve(hash);
                });
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async awaitTransactionConfirmation(web3Promise) {
        return new Promise(async (resolve, reject) => {
            let transactionHash = await this.getTransactionHash(web3Promise);
            let interval = setInterval(async () => {
                let receipt = await this.web3.eth.getTransactionReceipt(transactionHash);
                if (!receipt) {
                    return;
                }
                if (receipt.status == true) {
                    resolve(receipt);
                }
                else {
                    reject(new Error('Transaction was reverted'));
                }
                clearInterval(interval);
            }, 3000);
        });
    }
}
exports.Web3Adapter = Web3Adapter;
//# sourceMappingURL=web3-adapter.js.map