"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const ethereum_service_1 = require("./ethereum-service");
const encryption_service_1 = require("./encryption-service");
const data_packet_service_1 = require("./data-packet-service");
class BackendService {
    constructor(backendHost, ethereumService) {
        this.backendHost = backendHost;
        this.ethereumService = ethereumService;
    }
    async getDataPackets(owner) {
        return new Promise(async (resolve, reject) => {
            try {
                let response = await axios_1.default({
                    method: 'get',
                    url: this.backendHost + '/data-packets',
                    params: {
                        owner: owner
                    }
                });
                let dataPacketIds = response.data;
                let promises = dataPacketIds.map(async (id) => {
                    let packetData = await this.ethereumService.getPacket(id);
                    return new data_packet_service_1.Packet(packetData.id.toLowerCase(), packetData.cmw, packetData.materialId, packetData.publicFilesAddresses, packetData.privateFilesAddresses, packetData.status);
                });
                Promise.all(promises).then(dataPackets => {
                    resolve(dataPackets);
                });
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async registerEntity(entity, publicKey) {
        return new Promise(async (resolve, reject) => {
            try {
                await axios_1.default.post(this.backendHost + '/register-entity', {
                    entity: entity,
                    publicKey: publicKey.toString(),
                    publicKeyHash: ethereum_service_1.EthereumService.shasum(publicKey)
                });
                resolve(entity);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async getEntity(entityId) {
        return new Promise(async (resolve, reject) => {
            try {
                let response = await axios_1.default.get(this.backendHost + '/entity', {
                    params: {
                        id: entityId
                    }
                });
                resolve(response.data);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async getEntities(type) {
        return new Promise(async (resolve, reject) => {
            try {
                let response = await axios_1.default.get(this.backendHost + '/entities', {
                    params: {
                        type: type
                    }
                });
                resolve(response.data);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async getEntityPublicKey(id) {
        return new Promise(async (resolve, reject) => {
            try {
                let response = await axios_1.default.get(this.backendHost + '/entity-public-key', {
                    params: {
                        id: id
                    }
                });
                let publicKey = Buffer.from(response.data.publicKey);
                resolve(publicKey);
            }
            catch (error) {
                let input = prompt('Please provide your public key (a nice UI will handle this later):');
                if (input) {
                    let publicKey = Buffer.from(input);
                    if (!publicKey) {
                        reject(error);
                    }
                    resolve(publicKey);
                }
                else {
                    reject(new Error('Invalid public key'));
                }
            }
        });
    }
    async registerDataPacketEnvelopeAddress(ownerAddress, dataPacketEnvelopeAddress, privateKey) {
        let signature = await encryption_service_1.EncryptionService.signWithPrivateKey(Buffer.from(dataPacketEnvelopeAddress), privateKey);
        await axios_1.default.post(this.backendHost + '/register-data-packet-envelope', {
            dataPacketEnvelope: {
                owner: ownerAddress,
                dataPacketEnvelopeAddress: dataPacketEnvelopeAddress
            },
            signature: signature
        });
        return dataPacketEnvelopeAddress;
    }
    async transferDataPacketEnvelopeAddress(currentOwnerAddress, currentOwnerAddressPrivateKey, newOwnerAddress, dataPacketEnvelopeAddress) {
        let signature = await encryption_service_1.EncryptionService.signWithPrivateKey(Buffer.from(dataPacketEnvelopeAddress), currentOwnerAddressPrivateKey);
        await axios_1.default.post(this.backendHost + '/transfer-data-packet-envelope', {
            currentOwnerAddress: currentOwnerAddress,
            dataPacketEnvelope: {
                owner: newOwnerAddress,
                dataPacketEnvelopeAddress: dataPacketEnvelopeAddress
            },
            signature: signature
        });
        return dataPacketEnvelopeAddress;
    }
    async getDataPacketEnvelopeAddresses(owner) {
        return new Promise(async (resolve, reject) => {
            try {
                let response = await axios_1.default({
                    method: 'get',
                    url: this.backendHost + '/data-packet-envelopes',
                    params: {
                        owner: owner
                    }
                });
                resolve(response.data);
            }
            catch (error) {
                reject(error);
            }
        });
    }
}
exports.BackendService = BackendService;
//# sourceMappingURL=backend-service.js.map