"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const encryption_service_1 = require("./encryption-service");
const ethereum_service_1 = require("./ethereum-service");
class KeyPacket {
    constructor(aesKey, filesAddresses) {
        this.aesKey = aesKey;
        this.filesAddresses = filesAddresses;
    }
    static fromJson(json) {
        let parsedJson = JSON.parse(json);
        let aesKey = new encryption_service_1.AESKey(Buffer.from(parsedJson.aesKey.key.data), Buffer.from(parsedJson.aesKey.iv.data));
        return new KeyPacket(aesKey, parsedJson.filesAddresses);
    }
}
exports.KeyPacket = KeyPacket;
class TrackingData {
    constructor(cmw, rawAmount, grade, mineLatitude, mineLongitude, materialId) {
        this.cmw = cmw;
        this.rawAmount = rawAmount;
        this.grade = grade;
        this.mineLatitude = mineLatitude;
        this.mineLongitude = mineLongitude;
        this.materialId = materialId;
        if (cmw <= 0) {
            throw new Error('The CMW (Calculated Material Weight) needs to be greater than zero');
        }
        if (rawAmount <= 0) {
            throw new Error('The raw amount needs to be greater than zero');
        }
        if (grade <= 0) {
            throw new Error('The grade needs to be greater than zero');
        }
    }
}
exports.TrackingData = TrackingData;
class Packet {
    constructor(id, cmw, materialId, publicFilesAddresses, privateFilesAddresses, status) {
        this.id = id;
        this.cmw = cmw;
        this.materialId = materialId;
        this.publicFilesAddresses = publicFilesAddresses;
        this.privateFilesAddresses = privateFilesAddresses;
        this.status = status;
    }
}
exports.Packet = Packet;
class DataPacketEnvelope {
    constructor(packetId, publicFilesKey, privateFilesKey) {
        this.packetId = packetId;
        this.publicFilesKey = publicFilesKey;
        this.privateFilesKey = privateFilesKey;
    }
    toJson() {
        return JSON.stringify({
            packetId: this.packetId.toLowerCase(),
            publicFilesKey: {
                key: this.publicFilesKey.key,
                iv: this.publicFilesKey.iv
            },
            privateFilesKey: {
                key: this.privateFilesKey.key,
                iv: this.privateFilesKey.iv
            }
        });
    }
    static fromJson(rawDataPacketEnvelope) {
        return new DataPacketEnvelope(rawDataPacketEnvelope.packetId.toLowerCase(), new encryption_service_1.AESKey(Buffer.from(rawDataPacketEnvelope.publicFilesKey.key.data), Buffer.from(rawDataPacketEnvelope.publicFilesKey.iv.data)), new encryption_service_1.AESKey(Buffer.from(rawDataPacketEnvelope.privateFilesKey.key.data), Buffer.from(rawDataPacketEnvelope.privateFilesKey.iv.data)));
    }
}
exports.DataPacketEnvelope = DataPacketEnvelope;
class DataPacketService {
    constructor(fileStore, ethereumService, backendService) {
        this.fileStore = fileStore;
        this.ethereumService = ethereumService;
        this.backendService = backendService;
    }
    async certifyDataPacket(trackingData, publicFiles, privateFiles) {
        return new Promise(async (resolve, reject) => {
            try {
                let packetId = ethereum_service_1.EthereumService.generateRandomHex(20);
                let trackingDataFile = new File([JSON.stringify(trackingData)], 'minespider-tracking.json');
                let temporaryFilesKeyPacket = await this.uploadFiles(publicFiles);
                let publicFilesKeyPacket = await this.uploadFiles([trackingDataFile], temporaryFilesKeyPacket.aesKey);
                let privateFilesKeyPacket = await this.uploadFiles(privateFiles);
                let dataPacketEnvelope = new DataPacketEnvelope(packetId, publicFilesKeyPacket.aesKey, privateFilesKeyPacket.aesKey);
                let publicKey = await this.backendService.getEntityPublicKey(await this.ethereumService.getCurrentAccountAddress());
                await this.ethereumService.certifyDataPacket(packetId, trackingData.cmw, trackingData.materialId, temporaryFilesKeyPacket.filesAddresses.concat(publicFilesKeyPacket.filesAddresses), privateFilesKeyPacket.filesAddresses);
                resolve(await this.createEncryptedPacketEnvelope(dataPacketEnvelope, publicKey));
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async sellDataPacket(dataPacketEnvelope, cmw, newOwnerAddress, publicFiles, privateFiles, currentOwnerPrivateKey) {
        return new Promise(async (resolve, reject) => {
            try {
                let packet = await this.getPacket(dataPacketEnvelope.packetId);
                if (parseInt(String(packet.cmw)) < parseInt(String(cmw))) {
                    throw new Error('The CMW of the packet must be equal or bigger than the CMW to sell');
                }
                let newPacketId = ethereum_service_1.EthereumService.generateRandomHex(20);
                let publicFilesKeyPacket = await this.uploadFiles(publicFiles, dataPacketEnvelope.publicFilesKey);
                let publicFilesAddresses = packet.publicFilesAddresses.concat(publicFilesKeyPacket.filesAddresses);
                let privateFilesKeyPacket = await this.uploadFiles(privateFiles);
                if (parseInt(String(packet.cmw)) == parseInt(String(cmw))) {
                    await this.ethereumService.sellPacket(dataPacketEnvelope.packetId, newPacketId, newOwnerAddress, cmw, publicFilesAddresses, privateFilesKeyPacket.filesAddresses);
                    resolve(newPacketId);
                }
                else {
                    let remainingPacketId = ethereum_service_1.EthereumService.generateRandomHex(20);
                    await this.ethereumService.splitAndSellPacket(dataPacketEnvelope.packetId, newOwnerAddress, remainingPacketId, newPacketId, cmw, publicFilesAddresses, privateFilesKeyPacket.filesAddresses);
                    let remainingDataPacketEnvelope = new DataPacketEnvelope(remainingPacketId, dataPacketEnvelope.publicFilesKey, dataPacketEnvelope.privateFilesKey);
                    let currentOwnerAddress = await this.ethereumService.getCurrentAccountAddress();
                    let currentOwnerPublicKey = await this.backendService.getEntityPublicKey(currentOwnerAddress);
                    let remainingDataPacketEnvelopeAddress = await this.createEncryptedPacketEnvelope(remainingDataPacketEnvelope, currentOwnerPublicKey);
                    await this.backendService.registerDataPacketEnvelopeAddress(await this.ethereumService.getCurrentAccountAddress(), remainingDataPacketEnvelopeAddress, currentOwnerPrivateKey);
                }
                let newDataPacketEnvelope = new DataPacketEnvelope(newPacketId, dataPacketEnvelope.publicFilesKey, privateFilesKeyPacket.aesKey);
                let newOwnerPublicKey = await this.backendService.getEntityPublicKey(newOwnerAddress);
                let newDataPacketEnvelopeAddress = await this.createEncryptedPacketEnvelope(newDataPacketEnvelope, newOwnerPublicKey);
                await this.backendService.transferDataPacketEnvelopeAddress(await this.ethereumService.getCurrentAccountAddress(), currentOwnerPrivateKey, newOwnerAddress, newDataPacketEnvelopeAddress);
                resolve(newDataPacketEnvelopeAddress);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async createEncryptedPacketEnvelope(packetEnvelope, publicKey) {
        let encryptedPacketEnvelope = await encryption_service_1.EncryptionService.encryptWithPublicKey(Buffer.from(packetEnvelope.toJson()), publicKey);
        let file = new File([Buffer.from(encryptedPacketEnvelope.toString())], ethereum_service_1.EthereumService.shasum(encryptedPacketEnvelope));
        return await this.fileStore.save(file);
    }
    async downloadDataPacketEnvelope(dataPacketEnvelopeAddress, privateKey) {
        return new Promise(async (resolve, reject) => {
            try {
                let encryptedDataPacketEnvelope = await this.fileStore.get(dataPacketEnvelopeAddress);
                let decryptedDataPacketEnvelopeBuffer = await encryption_service_1.EncryptionService.decryptWithPrivateKey(encryptedDataPacketEnvelope, privateKey);
                let decryptedDataPacketEnvelope = JSON.parse(decryptedDataPacketEnvelopeBuffer.toString());
                let dataPacketEnvelope = DataPacketEnvelope.fromJson(decryptedDataPacketEnvelope);
                resolve(dataPacketEnvelope);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async getPacket(packetId) {
        return this.ethereumService.getPacket(packetId);
    }
    async getMyDataPackets() {
        return this.getDataPackets(await this.ethereumService.getCurrentAccountAddress());
    }
    async getDataPackets(owner) {
        return this.backendService.getDataPackets(owner);
    }
    async uploadFiles(files, aesKey) {
        return new Promise(async (resolve, reject) => {
            try {
                if (files.length === 0) {
                    throw new Error('At least one payload is required to generate the Key packet');
                }
                if (!aesKey) {
                    aesKey = encryption_service_1.EncryptionService.generateAESKey();
                }
                let addresses = new Array();
                for (let i = 0; i < files.length; i++) {
                    let file = files[i];
                    if (file) {
                        let fileContent = await this.fileToBuffer(file);
                        let fileContentHex = Buffer.from(fileContent.toString('hex'));
                        let encryptedFileContent = encryption_service_1.EncryptionService.encrypt(fileContentHex, aesKey);
                        let fileHash = await this.fileStore.save(new File([encryptedFileContent], ethereum_service_1.EthereumService.shasum(Buffer.from(file.name))));
                        let metadata = {
                            filename: file.name,
                            type: file.type,
                            target: fileHash
                        };
                        let encryptedFileMetadata = encryption_service_1.EncryptionService.encrypt(Buffer.from(JSON.stringify(metadata)), aesKey);
                        let address = await this.fileStore.save(new File([encryptedFileMetadata], ethereum_service_1.EthereumService.shasum(Buffer.from(file.name))));
                        addresses.push(address);
                    }
                }
                let keyPacket = new KeyPacket(aesKey, addresses);
                resolve(keyPacket);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async downloadMetadataFiles(keyPacket) {
        return new Promise(async (resolve, reject) => {
            try {
                let metadataFiles = new Map();
                for (let address of keyPacket.filesAddresses) {
                    let encryptedMetadataFile = await this.fileStore.get(address);
                    let metadataFileContent = encryption_service_1.EncryptionService.decrypt(encryptedMetadataFile.toString(), keyPacket.aesKey);
                    let metadata = JSON.parse(metadataFileContent.toString());
                    metadataFiles.set(address, metadata);
                }
                resolve(metadataFiles);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async downloadFile(key, address) {
        return new Promise(async (resolve, reject) => {
            try {
                let encryptedMetadataFile = await this.fileStore.get(address);
                let metadataFileContent = encryption_service_1.EncryptionService.decrypt(encryptedMetadataFile.toString(), key);
                let metadata = JSON.parse(metadataFileContent.toString());
                let encryptedFile = await this.fileStore.get(metadata.target);
                let decryptedContentHex = encryption_service_1.EncryptionService.decrypt(encryptedFile.toString(), key).toString();
                let decryptedContent = Buffer.from(decryptedContentHex, 'hex');
                resolve(new File([decryptedContent], metadata.filename, {
                    type: metadata.type
                }));
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async downloadFiles(keyPacket) {
        return new Promise(async (resolve, reject) => {
            try {
                let files = [];
                for (let address of keyPacket.filesAddresses) {
                    files.push(await this.downloadFile(keyPacket.aesKey, address));
                }
                resolve(files);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    async fileToBuffer(file) {
        return new Promise(resolve => {
            let reader = new FileReader();
            reader.onload = (event) => {
                if (!event.target.result) {
                    throw new Error('The result is missing');
                }
                resolve(Buffer.from(event.target.result));
            };
            reader.readAsArrayBuffer(file);
        });
    }
    async decryptKeyPacket(encryptedKeyPacket, aesKey) {
        return new Promise(async (resolve, reject) => {
            try {
                let content = await encryption_service_1.EncryptionService.decrypt(encryptedKeyPacket.toString(), aesKey);
                resolve(new File([content], ethereum_service_1.EthereumService.shasum(content)));
            }
            catch (error) {
                reject(error);
            }
        });
    }
}
exports.DataPacketService = DataPacketService;
//# sourceMappingURL=data-packet-service.js.map